@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-#  Copyright 2018-2022 Ramil Nugmanov <nougmanoff@protonmail.com>
+#  Copyright 2018-2023 Ramil Nugmanov <nougmanoff@protonmail.com>
 #  Copyright 2021 Dmitrij Zanadvornykh <zandmitrij@gmail.com>
 #  Copyright 2018 Tagir Akhmetshin <tagirshin@gmail.com>
 #  This file is part of chython.
@@ -49,6 +49,8 @@ class Standardize:
         s = self.standardize(_fix_stereo=False, logging=True, ignore=ignore, fix_tautomers=fix_tautomers)
         h, changed = self.implicify_hydrogens(_fix_stereo=False, logging=True)
 
+        if fix_tautomers and (logging or keep_kekule):  # thiele can change tautomeric form
+            hgs = self._hydrogens.copy()
         if keep_kekule:  # save bond orders
             bonds = [(b, b.order) for _, _, b in self.bonds()]
 
@@ -56,10 +58,11 @@ class Standardize:
         c = self.standardize_charges(prepare_molecule=False, logging=True)
 
         if keep_kekule and t:  # restore
-            if c:  # we need to do full kekule again
-                self.kekule()
+            # check ring charge/hydrogen moving
+            if c or fix_tautomers and hgs != self._hydrogens:  # noqa
+                self.kekule()  # we need to do full kekule again
             else:
-                for b, o in bonds:
+                for b, o in bonds:  # noqa
                     b._Bond__order = o  # noqa
             self.flush_cache()
 
@@ -70,8 +73,16 @@ class Standardize:
                 s.append((tuple(changed), -1, 'implicified'))
             if t:
                 s.append(((), -1, 'aromatized'))
+                if fix_tautomers and hgs != self._hydrogens:
+                    s.append((tuple(x for x, y in self._hydrogens.items() if hgs[x] != y),
+                              -1, 'aromatic tautomer found'))
             if c:
                 s.append((tuple(c), -1, 'recharged'))
+            if keep_kekule and t:
+                if c or fix_tautomers and hgs != self._hydrogens:
+                    s.append(((), -1, 'kekulized again'))
+                else:
+                    s.append(((), -1, 'kekule form restored'))
             return s
         return bool(k or s or h or t or c)
 
@@ -224,41 +235,29 @@ class Standardize:
             return []
         return False
 
-    def remove_hydrogen_bonds(self: 'MoleculeContainer', *, keep_to_terminal=True, _fix_stereo=True) -> int:
-        """Remove hydrogen bonds marked with 8 (any) bond
+    def remove_coordinate_bonds(self: 'MoleculeContainer', *, keep_to_terminal=True, _fix_stereo=True) -> int:
+        """Remove coordinate (or hydrogen) bonds marked with 8 (any) bond
 
         :param keep_to_terminal: Keep any bonds to terminal hydrogens
         :return: removed bonds count
         """
         bonds = self._bonds
-        hg = defaultdict(set)
-        for n, atom in self._atoms.items():
-            if atom.atomic_number == 1:
-                for m, b in bonds[n].items():
-                    if b.order == 8:
-                        hg[n].add(m)
+
+        ab = [(n, m) for n, m, b in self.bonds() if b.order == 8]
 
         if keep_to_terminal:
-            for n, ms in hg.items():
-                if len(bonds[n]) == len(ms):  # H~A or A~H~A etc case
-                    m = ms.pop()
-                    if m in hg:  # H~H case
-                        hg[m].discard(n)
+            skeleton = self.not_special_connectivity
+            hs = {n for n, a in self._atoms.items() if a.atomic_number == 1 and not skeleton[n]}
+            ab = [(n, m) for n, m in ab if n not in hs and m not in hs]
 
-        seen = set()
-        c = 0
-        for n, ms in hg.items():
-            seen.add(n)
-            for m in ms:
-                if m in seen:
-                    continue
-                c += 1
-                del bonds[n][m], bonds[m][n]
-        if c:
+        for n, m in ab:
+            del bonds[n][m], bonds[m][n]
+
+        if ab:
             self.flush_cache()
             if _fix_stereo:
                 self.fix_stereo()
-        return c
+        return len(ab)
 
     def implicify_hydrogens(self: 'MoleculeContainer', *, logging=False, _fix_stereo=True) -> \
             Union[int, Tuple[int, List[int]]]:
@@ -276,22 +275,21 @@ class Standardize:
         plane = self._plane
         hydrogens = self._hydrogens
         parsed_mapping = self._parsed_mapping
-        calc_implicit = self._calc_implicit
 
         explicit = defaultdict(list)
         for n, atom in atoms.items():
             if atom.atomic_number == 1 and (atom.isotope is None or atom.isotope == 1):
                 if len(bonds[n]) > 1:
-                    raise ValenceError(f'Hydrogen atom {{{n}}} has invalid valence. Try to use remove_hydrogen_bonds()')
+                    raise ValenceError(f'Hydrogen atom {n} has invalid valence. Try to use remove_coordinate_bonds()')
                 for m, b in bonds[n].items():
                     if b.order == 1:
                         if atoms[m].atomic_number != 1:  # not H-H
                             explicit[m].append(n)
                     elif b.order != 8:
-                        raise ValenceError(f'Hydrogen atom {{{n}}} has invalid valence {{{b.order}}}.')
+                        raise ValenceError(f'Hydrogen atom {n} has invalid valence {b.order}.')
 
         to_remove = set()
-        fixed = []
+        fixed = {}
         for n, hs in explicit.items():
             atom = atoms[n]
             charge = charges[n]
@@ -309,11 +307,14 @@ class Standardize:
                     rules = atom.valence_rules(charge, is_radical, explicit_sum)
                 except ValenceError:
                     break
-                if any(s.issubset(explicit_dict) and all(explicit_dict[k] >= c for k, c in d.items()) and h >= i
-                       for s, d, h in rules):
-                    to_remove.update(hi)
-                    fixed.append(n)
-                    break
+                for s, d, h in rules:
+                    if s.issubset(explicit_dict) and all(explicit_dict[k] >= c for k, c in d.items()) and h == i:
+                        to_remove.update(hi)
+                        fixed[n] = i
+                        break
+                else:
+                    continue
+                break
 
         for n in to_remove:
             del atoms[n]
@@ -328,8 +329,8 @@ class Standardize:
             except KeyError:
                 pass
 
-        for n in fixed:
-            calc_implicit(n)
+        for n, h in fixed.items():
+            hydrogens[n] = h
 
         if to_remove:
             self.flush_cache()
@@ -338,7 +339,7 @@ class Standardize:
                 self.fix_stereo()
 
         if logging:
-            return len(to_remove), fixed
+            return len(to_remove), list(fixed)
         return len(to_remove)
 
     def explicify_hydrogens(self: 'MoleculeContainer', *, start_map=None, _return_map=False, _fix_stereo=True) -> \
@@ -354,7 +355,7 @@ class Standardize:
             try:
                 to_add.extend([n] * h)
             except TypeError:
-                raise ValenceError(f'atom {{{n}}} has valence error')
+                raise ValenceError(f'atom {n} has valence error')
 
         if to_add:
             log = []
