@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-#  Copyright 2021, 2022 Ramil Nugmanov <nougmanoff@protonmail.com>
+#  Copyright 2021-2023 Ramil Nugmanov <nougmanoff@protonmail.com>
 #  This file is part of chython.
 #
 #  chython is free software; you can redistribute it and/or modify
@@ -16,34 +16,34 @@
 #  You should have received a copy of the GNU Lesser General Public License
 #  along with this program; if not, see <https://www.gnu.org/licenses/>.
 #
+from abc import ABCMeta, abstractmethod
 from base64 import urlsafe_b64encode
 from fileinput import FileInput
 from io import StringIO, TextIOWrapper
 from itertools import islice
 from os.path import abspath, join
 from pathlib import Path
-from pickle import dump, load, UnpicklingError
+from pickle import load, UnpicklingError
 from sys import platform
 from tempfile import gettempdir
-from typing import Union, Iterator, List
-from .stereo import MDLStereo
+from typing import Union, Iterator, List, Dict, Optional
 from ...containers import ReactionContainer, MoleculeContainer
-from ...exceptions import ParseError
 
 
-class MDLReadMeta(type):
+class MDLReadMeta(ABCMeta):
     def __call__(cls, *args, **kwargs):
         if kwargs.get('indexable'):
-            _cls = type(cls.__name__, (cls,), {'__len__': lambda x: len(x._shifts) - 1, '__module__': cls.__module__})
-            obj = object.__new__(_cls)
+            _cls = type(cls.__name__, (cls,), {'__len__': lambda x: len(x._shifts), '__module__': cls.__module__})
+            obj = object.__new__(_cls)  # noqa
         else:
-            obj = object.__new__(cls)
+            obj = object.__new__(cls)  # noqa
         obj.__init__(*args, **kwargs)
         return obj
 
 
-class MDLRead(MDLStereo, metaclass=MDLReadMeta):
-    def __init__(self, file, **kwargs):
+class MDLRead(metaclass=MDLReadMeta):
+    def __init__(self, file, buffer_size=1000, indexable=False, ignore=True, remap=False, ignore_bad_isotopes=False,
+                 ignore_stereo=False, calc_cis_trans=False):
         if isinstance(file, str):
             self._file = open(file)
             self._is_buffer = False
@@ -54,10 +54,44 @@ class MDLRead(MDLStereo, metaclass=MDLReadMeta):
             self._file = file
             self._is_buffer = True
         else:
-            raise TypeError('invalid file. TextIOWrapper, StringIO subclasses possible')
-        super().__init__(**kwargs)
+            raise TypeError('invalid file. TextIOWrapper, StringIO or FileInput subclasses or path to file expected')
+        self._shifts = None
+        self._tell = 0
+        self._buffer_size = buffer_size
+        self._buffer = None
+        self._ignore = ignore
+        self._remap = remap
+        self._ignore_bad_isotopes = ignore_bad_isotopes
+        self._ignore_stereo = ignore_stereo
+        self._calc_cis_trans = calc_cis_trans
+        if indexable:
+            self._load_cache()
 
-    def close(self, force=False):
+    def read(self, amount: Optional[int] = None) -> List[Union[ReactionContainer, MoleculeContainer]]:
+        """
+        Parse whole file
+
+        :param amount: number of records to read
+        """
+        if amount:
+            return list(islice(iter(self), amount))
+        return list(iter(self))
+
+    @abstractmethod
+    def read_structure(self, *, current: bool = True):
+        """
+        Read Reaction or Molecule container.
+
+        :param current: return current structure if already parsed, otherwise read next
+        """
+
+    @abstractmethod
+    def read_metadata(self, *, current: bool = True) -> Dict[str, str]:
+        """
+        Read metadata block
+        """
+
+    def close(self, force: bool = False):
         """
         Close opened file
 
@@ -66,11 +100,44 @@ class MDLRead(MDLStereo, metaclass=MDLReadMeta):
         if not self._is_buffer or force:
             self._file.close()
 
-    def __enter__(self):
-        return self
+    def tell(self):
+        """
+        Number of records processed from the original file
+        """
+        return self._tell
 
-    def __exit__(self, _type, value, traceback):
-        self.close()
+    def seek(self, offset):
+        """
+        Shift to a given record number
+        """
+        if self._shifts:
+            if 0 <= offset < len(self._shifts):
+                self._tell = offset
+                self._buffer = None
+                self._file.seek(self._shifts[offset])
+            else:
+                raise IndexError('invalid offset')
+        else:
+            raise NotImplementedError('Indexable supported in unix-like o.s. and for files stored on disk')
+
+    @abstractmethod
+    def reset_index(self):
+        """
+        Create (rewrite) indexation table. Implemented only for object that
+        is a real file (the path to the file is specified) because the external grep utility is used.
+        """
+
+    def read_block(self, *, current: bool = True) -> str:
+        """
+        Read full record block with metadata
+        """
+        return ''.join(self._read_block(current=current))
+
+    @abstractmethod
+    def _read_block(self, *, current: bool = True) -> List[str]:
+        """
+        Read full record block with metadata
+        """
 
     def _load_cache(self):
         """
@@ -79,112 +146,80 @@ class MDLRead(MDLStereo, metaclass=MDLReadMeta):
         if platform == 'win32' or self._is_buffer:
             return
         try:
-            with open(self.__cache_path, 'rb') as f:
+            with open(self._cache_path, 'rb') as f:
                 self._shifts = load(f)
         except FileNotFoundError:  # cache not found
             self.reset_index()
         except IsADirectoryError as e:
-            raise IsADirectoryError(f'Please delete {self.__cache_path} directory') from e
+            raise IsADirectoryError(f'Please delete {self._cache_path} directory') from e
         except (UnpicklingError, EOFError) as e:  # invalid file. ask user to check it.
-            raise UnpicklingError(f'Invalid cache file {self.__cache_path}. Please delete it') from e
-
-    def reset_index(self):
-        """
-        Create (rewrite) indexation table. Implemented only for object that
-        is a real file (the path to the file is specified) because the external grep utility is used.
-        """
-        if platform != 'win32' and not self._is_buffer:
-            self._shifts = self._get_shifts(self._file.name)
-            with open(self.__cache_path, 'wb') as f:
-                dump(self._shifts, f)
-        else:
-            raise self._implement_error
+            raise UnpicklingError(f'Invalid cache file {self._cache_path}. Please delete it') from e
 
     @property
-    def __cache_path(self):
+    def _cache_path(self):
         return abspath(join(gettempdir(), 'chython_' + urlsafe_b64encode(abspath(self._file.name).encode()).decode()))
 
-    def read(self) -> List[Union[ReactionContainer, MoleculeContainer]]:
-        """
-        Parse whole file
+    def __enter__(self):
+        return self
 
-        :return: list of parsed molecules
-        """
-        return list(iter(self))
+    def __exit__(self, _type, value, traceback):
+        self.close()
 
     def __iter__(self) -> Iterator[Union[ReactionContainer, MoleculeContainer]]:
-        return (x for x in self._data if not isinstance(x, ParseError))
+        while True:
+            try:
+                yield self.read_structure(current=False)
+            except ValueError:
+                pass
+            except EOFError:
+                return
 
     def __next__(self) -> Union[ReactionContainer, MoleculeContainer]:
         return next(iter(self))
 
-    def __getitem__(self, item) -> Union[ReactionContainer, MoleculeContainer, ParseError]:
+    def __getitem__(self, item) -> Union[ReactionContainer, MoleculeContainer,
+                                         List[Union[ReactionContainer, MoleculeContainer]]]:
         """
         Getting the item by index from the original file,
         For slices records with errors skipped.
-        For indexed access records with errors returned as error container.
-        :return: [Molecule, Reaction]Container or list of [Molecule, Reaction]Containers
         """
         if self._shifts:
-            _len = len(self._shifts) - 1
+            _len = len(self._shifts)
             if isinstance(item, int):
                 if item >= _len or item < -_len:
                     raise IndexError('List index out of range')
                 if item < 0:
                     item += _len
                 self.seek(item)
-                return next(self._data)
+                return self.read_structure()
             elif isinstance(item, slice):
                 start, stop, step = item.indices(_len)
                 if start == stop:
                     return []
                 if step == 1:
                     self.seek(start)
-                    records = [x for x in islice(self._data, stop - start) if not isinstance(x, ParseError)]
+                    records = []
+                    for _ in range(start, stop):
+                        try:
+                            records.append(self.read_structure(current=False))
+                        except EOFError:
+                            break
+                        except ValueError:
+                            pass
                 else:
                     records = []
                     for index in range(start, stop, step):
                         self.seek(index)
-                        record = next(self._data)
-                        if not isinstance(record, ParseError):
-                            records.append(record)
+                        try:
+                            records.append(self.read_structure())
+                        except EOFError:
+                            break
+                        except ValueError:
+                            pass
                 return records
             else:
                 raise TypeError('Indices must be integers or slices')
-        raise self._implement_error
-
-    def read_text(self, item):
-        """
-        Read record block as text
-        """
-        if self._shifts:
-            if not isinstance(item, int):
-                raise TypeError('int required')
-            _len = len(self._shifts) - 1
-            if item >= _len or item < -_len:
-                raise IndexError('List index out of range')
-            if item < 0:
-                item += _len
-            start = self._shifts[item]
-            end = self._shifts[item + 1]
-            current = self._file.tell()
-            self._file.seek(start)
-            data = self._file.read(end - start)
-            self._file.seek(current)
-            return data
-        raise self._implement_error
-
-    def _prepare_meta(self, meta):
-        new_meta = {}
-        for k, v in meta.items():
-            if v:
-                new_meta[k] = '\n'.join(v)
-            else:
-                self._info(f'invalid metadata entry: {k}: {v}')
-        return new_meta
-
-    _shifts = None
-    _implement_error = NotImplementedError('Indexable supported in unix-like o.s. and for files stored on disk')
+        raise NotImplementedError('Indexable supported in unix-like o.s. and for files stored on disk')
 
 
 __all__ = ['MDLRead']
